//Transit Warp
//Digital Slaves

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

float time ;
float seed ;

//texture
texture Tex <string uiname="Warp Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};
texture Tex2 <string uiname="Texture1";>;
sampler Samp2 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex2);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};
texture Tex3 <string uiname="Texture2";>;
sampler Samp3 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex3);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float4x4 tCol <string uiname="Color Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function

float4 Transit_Warp(float2 uv)
{

        float2 center = float2(0.5,0.5);
        float2 toUV = uv - center;
        float distanceFromCenter = length(toUV);
        float2 normToUV = toUV / distanceFromCenter;
        float angle = (atan2(normToUV.y, normToUV.x) + 3.141592) / (2.0 * 3.141592);
        float offset1 = tex2D(Samp, float2(angle, frac(time/3 + distanceFromCenter/5 + seed))).x * 2.0 - 1.0;
        float offset2 = offset1 * 2.0 * min(0.3, (1-time)) * distanceFromCenter;
        offset1 = offset1 * 2.0 * min(0.3, time) * distanceFromCenter;

        float4 c1 = tex2D(Samp2, frac(center + normToUV * (distanceFromCenter + offset1)));
        float4 c2 = tex2D(Samp3, frac(center + normToUV * (distanceFromCenter + offset2)));

        return lerp(c1, c2, time);
}

struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------
float4 PS(float2 uv : TEXCOORD) : COLOR
{
return Transit_Warp(uv);
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique Transit_Warp
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_2_b PS();
    }
}
