//Transit Fuzzy
//Digital Slaves
// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

float time ;
float fuzzyAmount;

//texture
texture Tex <string uiname="Texture1";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};
texture Tex2 <string uiname="Texture2";>;
sampler Samp2 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex2);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};


float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float4x4 tCol <string uiname="Color Transform";>;

float4 Transit_Fuzzy(float2 uv)
{

       float radius = -fuzzyAmount + time * (0.70710678 + 2.0 * fuzzyAmount);
       float fromCenter = length(uv - float2(0.5,0.5));
       float distFromCircle = fromCenter - radius;

       float4 c1 = tex2D(Samp, uv);
       float4 c2 = tex2D(Samp2, uv);

       float p = saturate((distFromCircle + fuzzyAmount) / (2.0 * fuzzyAmount));

       return lerp(c2, c1, p);
}
//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function


struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(float2 uv : TEXCOORD) : COLOR

{

       return Transit_Fuzzy(uv);
       
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TransitFuzzy
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_2_0 PS();
    }
}

